require 'test/unit'
require File.dirname(__FILE__) + '/../lib/staticmatic'
require File.dirname(__FILE__) + '/test_helper'

class TestHelperTest < Test::Unit::TestCase
  include StaticMatic::Helpers

  def test_should_split_html_tag
    tags_to_test = [ {:name => 'br', :attributes => {} },
                     {:name => 'img', :attributes => {'src' => 'images/testImage.gif', 'alt' => 'Test Image'} },
                     {:name => 'textarea', :attributes => {'rows' => '10', 'cols' => '40'}, :content => 'Test data in here' },
                     {:name => 'a', :attributes => {'href' => '/test.html', 
                                                    'onclick' => "doStuff('text'); function {}; if 2 > 1 then return false;"},
                                    :content => 'Link to nowhere'}]
                     
    tags_to_test.each do |tag_parts|
      if !tag_parts[:content]
        tag_text = tag(tag_parts[:name], tag_parts[:attributes])
      else
        tag_text = tag(tag_parts[:name], tag_parts[:attributes]) { tag_parts[:content] }
      end
      split_tag = split_html_tag(tag_text)
      assert_equal(tag_parts[:name], split_tag[:name])
      assert_equal(tag_parts[:options], split_tag[:options])
      if tag_parts[:contents]
        assert_equal(tag_parts[:content], split_tag[:content])
      end
    end
  end
  
  def test_should_match_tags
    original_tags = [%q{<br/>}, %q{<img src="/images/test.gif" alt="Test"/>}, %q{<li class="nav" id="1">Home</li>}]
    test_tags = [%q{<br>}, %q{<img alt="Test" src="/images/test.gif"/>}, %q{<li id="1" class="nav">Home</li>}]
    original_tags.each_index do |index|
      assert_nothing_raised { assert_tags_match(original_tags[index], test_tags[index]) }
    end
  end

  def test_should_not_match_tags
    original_tags = [%q{<img src="/images/test.gif" alt="Test"/>}, %q{<li class="nav" id="1">Home</li>}, %q{<li>Home</li>}]
    test_tags = [%q{<img src="/images/test.gif"/>}, %q{<li id="2" class="nav">Home</li>}, %q{<li>Contact</li>}]
    original_tags.each_index do |index|
      assert_raise Test::Unit::AssertionFailedError do
        assert_tags_match(original_tags[index], test_tags[index])
      end
    end
  end

end
